@@ -6,21 +6,19 @@ import warnings
 import weakref
 from functools import cached_property, wraps
 from inspect import CO_VARARGS, isgeneratorfunction, unwrap
+from logging import getLogger
 from types import CodeType
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
     ContextManager,
-    Dict,
     Iterable,
-    List,
     Literal,
     Mapping,
     NamedTuple,
     Optional,
     Tuple,
-    Type,
     TypeVar,
     Union,
     cast,
@@ -30,6 +28,9 @@ from typing import (
 from ._type_resolution import _resolve_sig_or_inform, resolve_type_hints
 from ._util import _split_union, issubclassable
 
+logger = getLogger("in_n_out")
+
+
 if TYPE_CHECKING:
     from inspect import Signature
 
@@ -87,8 +88,8 @@ class _RegisteredCallback(NamedTuple):
 
 
 class _CachedMap(NamedTuple):
-    all: Dict[object, List[Union[Processor, Provider]]]
-    subclassable: Dict[type, List[Union[Processor, Provider]]]
+    all: dict[object, list[Processor | Provider]]
+    subclassable: dict[type, list[Processor | Provider]]
 
 
 class InjectionContext(ContextManager):
@@ -101,8 +102,8 @@ class InjectionContext(ContextManager):
         self,
         store: Store,
         *,
-        providers: Optional[ProviderIterable] = None,
-        processors: Optional[ProcessorIterable] = None,
+        providers: ProviderIterable | None = None,
+        processors: ProcessorIterable | None = None,
     ) -> None:
         self._disposers = []
         if providers is not None:
@@ -124,7 +125,7 @@ class Store:
     """A Store is a collection of providers and processors."""
 
     _NULL = _NullSentinel()
-    _instances: Dict[str, Store] = {}
+    _instances: dict[str, Store] = {}
 
     @classmethod
     def create(cls, name: str) -> Store:
@@ -156,7 +157,7 @@ class Store:
         return cls._instances[name]
 
     @classmethod
-    def get_store(cls, name: Optional[str] = None) -> Store:
+    def get_store(cls, name: str | None = None) -> Store:
         """Get a Store instance with the given `name`.
 
         Parameters
@@ -204,9 +205,9 @@ class Store:
 
     def __init__(self, name: str) -> None:
         self._name = name
-        self._providers: List[_RegisteredCallback] = []
-        self._processors: List[_RegisteredCallback] = []
-        self._namespace: Union[Namespace, Callable[[], Namespace], None] = None
+        self._providers: list[_RegisteredCallback] = []
+        self._processors: list[_RegisteredCallback] = []
+        self._namespace: Namespace | Callable[[], Namespace] | None = None
         self.on_unresolved_required_args: RaiseWarnReturnIgnore = "warn"
         self.on_unannotated_required_args: RaiseWarnReturnIgnore = "warn"
         self.guess_self: bool = True
@@ -226,7 +227,7 @@ class Store:
             del self._cached_provider_map
 
     @property
-    def namespace(self) -> Dict[str, object]:
+    def namespace(self) -> dict[str, object]:
         """Return namespace for type resolution, if this store has one.
 
         If no namespace is set, this will return an empty `dict`.
@@ -238,7 +239,7 @@ class Store:
         return dict(self._namespace)
 
     @namespace.setter
-    def namespace(self, ns: Union[Namespace, Callable[[], Namespace]]) -> None:
+    def namespace(self, ns: Namespace | Callable[[], Namespace]) -> None:
         self._namespace = ns
 
     # ------------------------- Callback registration ------------------------------
@@ -246,8 +247,8 @@ class Store:
     def register(
         self,
         *,
-        providers: Optional[ProviderIterable] = None,
-        processors: Optional[ProcessorIterable] = None,
+        providers: ProviderIterable | None = None,
+        processors: ProcessorIterable | None = None,
     ) -> InjectionContext:
         """Register multiple providers and/or processors at once.
 
@@ -293,7 +294,7 @@ class Store:
     def register_provider(
         self,
         provider: Provider,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
     ) -> InjectionContext:
         """Register `provider` as a provider of `type_hint`.
@@ -319,7 +320,7 @@ class Store:
     def register_processor(
         self,
         processor: Processor,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
     ) -> InjectionContext:
         """Register `processor` as a processor of `type_hint`.
@@ -353,7 +354,7 @@ class Store:
         self,
         func: ProviderVar,
         *,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
     ) -> ProviderVar:
         ...
@@ -363,18 +364,18 @@ class Store:
         self,
         func: Literal[None] = ...,
         *,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
     ) -> Callable[[ProviderVar], ProviderVar]:
         ...
 
     def mark_provider(
         self,
-        func: Optional[ProviderVar] = None,
+        func: ProviderVar | None = None,
         *,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
-    ) -> Union[Callable[[ProviderVar], ProviderVar], ProviderVar]:
+    ) -> Callable[[ProviderVar], ProviderVar] | ProviderVar:
         """Decorate `func` as a provider of its first parameter type.
 
         Note, If func returns `Optional[Type]`, it will be registered as a provider
@@ -418,7 +419,7 @@ class Store:
         self,
         func: ProcessorVar,
         *,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
     ) -> ProcessorVar:
         ...
@@ -428,18 +429,18 @@ class Store:
         self,
         func: Literal[None] = ...,
         *,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
     ) -> Callable[[ProcessorVar], ProcessorVar]:
         ...
 
     def mark_processor(
         self,
-        func: Optional[ProcessorVar] = None,
+        func: ProcessorVar | None = None,
         *,
-        type_hint: Optional[object] = None,
+        type_hint: object | None = None,
         weight: float = 0,
-    ) -> Union[Callable[[ProcessorVar], ProcessorVar], ProcessorVar]:
+    ) -> Callable[[ProcessorVar], ProcessorVar] | ProcessorVar:
         """Decorate `func` as a processor of its first parameter type.
 
         Parameters
@@ -480,8 +481,8 @@ class Store:
     # ------------------------- Callback retrieval ------------------------------
 
     def iter_providers(
-        self, type_hint: Union[object, Type[T]]
-    ) -> Iterable[Callable[[], Optional[T]]]:
+        self, type_hint: object | type[T]
+    ) -> Iterable[Callable[[], T | None]]:
         """Iterate over all providers of `type_hint`.
 
         Parameters
@@ -497,7 +498,7 @@ class Store:
         return self._iter_type_map(type_hint, self._cached_provider_map)
 
     def iter_processors(
-        self, type_hint: Union[object, Type[T]]
+        self, type_hint: object | type[T]
     ) -> Iterable[Callable[[T], Any]]:
         """Iterate over all processors of `type_hint`.
 
@@ -515,7 +516,7 @@ class Store:
 
     # ------------------------- Instance retrieval ------------------------------
 
-    def provide(self, type_hint: Union[object, Type[T]]) -> Optional[T]:
+    def provide(self, type_hint: object | type[T]) -> T | None:
         """Provide an instance of `type_hint`.
 
         This will iterate over all providers of `type_hint` and return the first
@@ -542,9 +543,10 @@ class Store:
         self,
         result: Any,
         *,
-        type_hint: Union[object, Type[T], None] = None,
+        type_hint: object | type[T] | None = None,
         first_processor_only: bool = False,
         raise_exception: bool = False,
+        _funcname: str = "",
     ) -> None:
         """Provide an instance of `type_`.
 
@@ -565,11 +567,19 @@ class Store:
         raise_exception : bool, optional
             If `True`, and a processor raises an exception, it will be raised
             and the remaining processors will not be invoked.
+        _funcname: str, optional
+            The name of the function that called this method.  This is used internally
+            for debugging
         """
         if type_hint is None:
             type_hint = type(result)
-        for processor in self.iter_processors(type_hint):  # type: ignore
+        _processors: Iterable[Callable] = self.iter_processors(type_hint)
+        logger.debug(
+            "Invoking processors on result %r from function %r", result, _funcname
+        )
+        for processor in _processors:
             try:
+                logger.debug("  P: %s", processor)
                 processor(result)
             except Exception as e:  # pragma: no cover
                 if raise_exception:
@@ -589,10 +599,10 @@ class Store:
         *,
         providers: bool = True,
         processors: bool = False,
-        localns: Optional[dict] = None,
-        on_unresolved_required_args: Optional[RaiseWarnReturnIgnore] = None,
-        on_unannotated_required_args: Optional[RaiseWarnReturnIgnore] = None,
-        guess_self: Optional[bool] = None,
+        localns: dict | None = None,
+        on_unresolved_required_args: RaiseWarnReturnIgnore | None = None,
+        on_unannotated_required_args: RaiseWarnReturnIgnore | None = None,
+        guess_self: bool | None = None,
     ) -> Callable[..., R]:
         ...
         # unfortunately, the best we can do is convert the signature to Callabe[..., R]
@@ -606,24 +616,24 @@ class Store:
         *,
         providers: bool = True,
         processors: bool = False,
-        localns: Optional[dict] = None,
-        on_unresolved_required_args: Optional[RaiseWarnReturnIgnore] = None,
-        on_unannotated_required_args: Optional[RaiseWarnReturnIgnore] = None,
-        guess_self: Optional[bool] = None,
+        localns: dict | None = None,
+        on_unresolved_required_args: RaiseWarnReturnIgnore | None = None,
+        on_unannotated_required_args: RaiseWarnReturnIgnore | None = None,
+        guess_self: bool | None = None,
     ) -> Callable[[Callable[P, R]], Callable[..., R]]:
         ...
 
     def inject(
         self,
-        func: Optional[Callable[P, R]] = None,
+        func: Callable[P, R] | None = None,
         *,
         providers: bool = True,
         processors: bool = False,
-        localns: Optional[dict] = None,
-        on_unresolved_required_args: Optional[RaiseWarnReturnIgnore] = None,
-        on_unannotated_required_args: Optional[RaiseWarnReturnIgnore] = None,
-        guess_self: Optional[bool] = None,
-    ) -> Union[Callable[..., R], Callable[[Callable[P, R]], Callable[..., R]]]:
+        localns: dict | None = None,
+        on_unresolved_required_args: RaiseWarnReturnIgnore | None = None,
+        on_unannotated_required_args: RaiseWarnReturnIgnore | None = None,
+        guess_self: bool | None = None,
+    ) -> Callable[..., R] | Callable[[Callable[P, R]], Callable[..., R]]:
         """Decorate `func` to inject dependencies at calltime.
 
         Assuming `providers` is True (the default), this will attempt retrieve
@@ -728,7 +738,7 @@ class Store:
                 return self.inject_processors(func) if processors else func
 
             # bail if there aren't any annotations at all
-            code: Optional[CodeType] = getattr(unwrap(func), "__code__", None)
+            code: CodeType | None = getattr(unwrap(func), "__code__", None)
             if (code and not code.co_argcount) and "return" not in getattr(
                 func, "__annotations__", {}
             ):
@@ -749,36 +759,61 @@ class Store:
             if sig is None:  # something went wrong, and the user was notified.
                 return func
 
+            _fname = getattr(func, "__qualname__", func)
+
             # get provider functions for each required parameter
             @wraps(func)
             def _exec(*args: P.args, **kwargs: P.kwargs) -> R:
                 # we're actually calling the "injected function" now
-
+                logger.debug(
+                    "Executing @injected %s%s with args: %r, kwargs: %r",
+                    _fname,
+                    sig,
+                    args,
+                    kwargs,
+                )
                 _sig = cast("Signature", sig)  # mypy thinks sig is still optional
 
-                # first, get and call the provider functions for each parameter type:
-                _kwargs: Dict[str, Any] = {}
-                for param in _sig.parameters.values():
-                    provided = self.provide(param.annotation)
-                    if provided is not None:
-                        _kwargs[param.name] = provided
-
                 # use bind_partial to allow the caller to still provide their own args
                 # if desired. (i.e. the injected deps are only used if not provided)
                 bound = _sig.bind_partial(*args, **kwargs)
                 bound.apply_defaults()
 
+                # first, get and call the provider functions for each parameter type:
+                _injected_names: set[str] = set()
+                for param in _sig.parameters.values():
+                    if param.name not in bound.arguments:
+                        provided = self.provide(param.annotation)
+                        if provided is not None:
+                            logger.debug(
+                                "  injecting %s: %s = %r",
+                                param.name,
+                                param.annotation,
+                                provided,
+                            )
+                            _injected_names.add(param.name)
+                            bound.arguments[param.name] = provided
+
                 # call the function with injected values
+                logger.debug(
+                    "  Calling %s with %r (injected %r)",
+                    _fname,
+                    bound.arguments,
+                    _injected_names,
+                )
                 try:
-                    result = func(**{**_kwargs, **bound.arguments})
+                    result = func(**bound.arguments)
                 except TypeError as e:
                     if "missing" not in e.args[0]:
                         raise  # pragma: no cover
                     # likely a required argument is still missing.
                     # show what was injected and raise
                     _argnames = (
-                        f"arguments: {set(_kwargs)!r}" if _kwargs else "NO arguments"
+                        f"arguments: {_injected_names!r}"
+                        if _injected_names
+                        else "NO arguments"
                     )
+                    logger.exception(e)
                     raise TypeError(
                         f"After injecting dependencies for {_argnames}, {e}"
                     ) from e
@@ -817,7 +852,7 @@ class Store:
         self,
         func: Callable[P, R],
         *,
-        type_hint: Union[object, Type[T], None] = None,
+        type_hint: object | type[T] | None = None,
         first_processor_only: bool = False,
         raise_exception: bool = False,
     ) -> Callable[P, R]:
@@ -828,7 +863,7 @@ class Store:
         self,
         func: Literal[None] = None,
         *,
-        type_hint: Union[object, Type[T], None] = None,
+        type_hint: object | type[T] | None = None,
         first_processor_only: bool = False,
         raise_exception: bool = False,
     ) -> Callable[[Callable[P, R]], Callable[P, R]]:
@@ -836,12 +871,12 @@ class Store:
 
     def inject_processors(
         self,
-        func: Optional[Callable[P, R]] = None,
+        func: Callable[P, R] | None = None,
         *,
-        type_hint: Union[object, Type[T], None] = None,
+        type_hint: object | type[T] | None = None,
         first_processor_only: bool = False,
         raise_exception: bool = False,
-    ) -> Union[Callable[[Callable[P, R]], Callable[P, R]], Callable[P, R]]:
+    ) -> Callable[[Callable[P, R]], Callable[P, R]] | Callable[P, R]:
         """Decorate a function to process its output.
 
         Variant of inject, but only injects processors (for the sake of more explicit
@@ -895,6 +930,7 @@ class Store:
                         type_hint=type_hint,
                         first_processor_only=first_processor_only,
                         raise_exception=raise_exception,
+                        _funcname=getattr(func, "__qualname__", str(func)),
                     )
                 return result
 
@@ -906,13 +942,15 @@ class Store:
 
     @cached_property
     def _cached_provider_map(self) -> _CachedMap:
+        logger.debug("Rebuilding provider map cache")
         return self._build_map(self._providers)
 
     @cached_property
     def _cached_processor_map(self) -> _CachedMap:
+        logger.debug("Rebuilding processor map cache")
         return self._build_map(self._processors)
 
-    def _build_map(self, registry: List[_RegisteredCallback]) -> _CachedMap:
+    def _build_map(self, registry: list[_RegisteredCallback]) -> _CachedMap:
         """Build a map of type hints to callbacks.
 
         This is the sorted and cached version of the map that will be used to resolve
@@ -920,8 +958,8 @@ class Store:
         of *all* provider/processor type hints, regardless of whether they can be used
         with `is_subclass`.  The second is a map of only "issubclassable" type hints.
         """
-        all_: Dict[object, List[_RegisteredCallback]] = {}
-        subclassable: Dict[type, List[_RegisteredCallback]] = {}
+        all_: dict[object, list[_RegisteredCallback]] = {}
+        subclassable: dict[type, list[_RegisteredCallback]] = {}
         for p in registry:
             if p.origin not in all_:
                 all_[p.origin] = []
@@ -942,7 +980,7 @@ class Store:
         return _CachedMap(all_out, subclassable_out)
 
     def _iter_type_map(
-        self, hint: Union[object, Type[T]], callback_map: _CachedMap
+        self, hint: object | type[T], callback_map: _CachedMap
     ) -> Iterable[Callable]:
         _all_types = callback_map.all
         _subclassable_types = callback_map.subclassable
@@ -967,10 +1005,9 @@ class Store:
     def _register_callbacks(
         self,
         callbacks: CallbackIterable,
-        providers: bool = True,
+        is_provider: bool = True,
     ) -> Disposer:
-
-        if providers:
+        if is_provider:
             reg = self._providers
             cache_map = "_cached_provider_map"
             check_callback: Callable[[Any], Callable] = _validate_provider
@@ -979,7 +1016,7 @@ class Store:
                 "registration). Cannot be a provider."
             )
 
-            def _type_from_hints(hints: Dict[str, Any]) -> Any:
+            def _type_from_hints(hints: dict[str, Any]) -> Any:
                 return hints.get("return")
 
         else:
@@ -991,7 +1028,7 @@ class Store:
                 "at registration). Cannot be a processor."
             )
 
-            def _type_from_hints(hints: Dict[str, Any]) -> Any:
+            def _type_from_hints(hints: dict[str, Any]) -> Any:
                 return next((v for k, v in hints.items() if k != "return"), None)
 
         _callbacks: Iterable[CallbackTuple]
@@ -1000,10 +1037,11 @@ class Store:
         else:
             _callbacks = callbacks
 
-        to_register: List[_RegisteredCallback] = []
+        regname = "provider" if is_provider else "processor"
+        to_register: list[_RegisteredCallback] = []
         for tup in _callbacks:
             callback, *rest = tup
-            type_: Optional[HintArg] = None
+            type_: HintArg | None = None
             weight: float = 0
             if rest:
                 if len(rest) == 1:
@@ -1035,7 +1073,6 @@ class Store:
                     try:
                         hash(origin)
                     except TypeError:
-                        regname = "provider" if providers else "processor"
                         raise TypeError(
                             f"{origin!r} cannot be used as a {regname} hint, since it "
                             "is not hashable and cannot be passed as the second "
@@ -1043,12 +1080,23 @@ class Store:
                         ) from None
 
                 cb = _RegisteredCallback(origin, callback, is_opt, weight, subclassable)
+                logger.debug(
+                    "Registering %s of %s: %s (weight: %s, subclassable: %s)",
+                    regname,
+                    origin,
+                    callback,
+                    weight,
+                    subclassable,
+                )
                 to_register.append(cb)
 
         def _dispose() -> None:
             for p in to_register:
                 with contextlib.suppress(ValueError):
                     reg.remove(p)
+                    logger.debug(
+                        "Unregistering %s of %s: %s", regname, p.origin, p.callback
+                    )
             # attribute error in case the cache was never built
             with contextlib.suppress(AttributeError):
                 delattr(self, cache_map)
@@ -1062,7 +1110,7 @@ class Store:
         return _dispose
 
     def _methodwrap(
-        self, callback: types.MethodType, reg: List[_RegisteredCallback], cache_map: str
+        self, callback: types.MethodType, reg: list[_RegisteredCallback], cache_map: str
     ) -> Callable:
         """Wrap a method in a weakref to prevent a strong reference to the owner."""
         ref = weakref.WeakMethod(callback)
@@ -1084,7 +1132,7 @@ class Store:
         return _callback
 
 
-def _validate_provider(obj: Union[T, Callable[[], T]]) -> Callable[[], T]:
+def _validate_provider(obj: T | Callable[[], T]) -> Callable[[], T]:
     """Check that an object is a valid provider.
 
     Can either be a function or an object. If a non-callable is passed, a function
@@ -1101,7 +1149,7 @@ def _validate_processor(obj: Callable[[T], Any]) -> Callable[[T], Any]:
     """
     if not callable(obj):
         raise ValueError(f"Processors must be callable. Got {obj!r}")
-    co: Optional[CodeType] = getattr(obj, "__code__", None)
+    co: CodeType | None = getattr(obj, "__code__", None)
     if not co:
         # if we don't have a code object, we can't check the number of arguments
         # TODO: see if we can do better in the future, but better to just return
